/***************************************************************************
 *   Copyright (C) 2004 by Stephen Anthony                                 *
 *   stephena@users.sourceforge.net                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

using namespace std;

#include <iostream>

#include <kfiledialog.h>
#include <kstdaccel.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kinstance.h>
#include <kaboutdata.h>
#include <kapplication.h>
#include <kmainwindow.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <ktoolbar.h>
#include <kurl.h>
#include <krun.h>
#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kkeydialog.h>
#include <qwidget.h>
#include <qdir.h>
#include <qstrlist.h>
#include <qsplitter.h>
#include <qmessagebox.h>
#include <qlayout.h>

#include "kstella.h"
#include "game.h"
#include "gamelist.h"
#include "gamelistdoc.h"
#include "gamelistview.h"
#include "setup.h"
#include "settings.h"


int KStella::_romView = ALL;

KStella::KStella( void )
{
	config = kapp->config();

	// call inits to invoke all other construction parts
	initActions();
	initView();

	// create the game list and let the user view it
	doc = new GameListDoc( this );
	doc->load();
	view->setGameList( doc->gameList() );

	// change tabs when the left or right keys are pressed
	connect( view, SIGNAL(changeTab(int)), preview, SLOT(slotChangeTab(int)) );
	// update preview when a new game is selected
	connect( view, SIGNAL(changeSelection(Game*)), preview, SLOT(slotSetCurrentGame(Game*)) );
	// update status bar with game notes when a new game is selected
	connect( view, SIGNAL(changeSelection(Game*)), this, SLOT(slotChangeNote(Game*)) );
	// start emulation when game is double-clicked or return pressed
	connect( view, SIGNAL(startEmulation()), this, SLOT(slotEmulate()) );

	// read the Rom View option of ALL, AVAILABLE or UNAVAILABLE
	config->setGroup( "General Options" );
	int romView = config->readNumEntry( "Rom View", -1 );
	switch( romView )
	{
		case KStella::ALL:
			slotViewAllRoms();
			break;

		case KStella::AVAILABLE:
			slotViewAvailableRoms();
			break;

		case KStella::UNAVAILABLE:
			slotViewUnavailableRoms();
			break;

		default:
			slotViewAllRoms();
			break;
	}

	// create status bar for showing game notes
	statusBar()->show();

	// apply the saved mainwindow settings, if any, and ask the mainwindow
	// to automatically save settings if changed: window size, toolbar
	// position, icon size, etc.
	setAutoSaveSettings();
}


KStella::~KStella( void )
{
	saveOptions();
}


void KStella::initActions()
{
	// File menu
	(void) new KAction( i18n("Statistics"), 0, 0,
						this, SLOT(slotShowStatistics()),
						actionCollection(), "statistics");
	(void) new KAction( i18n("Stella documentation"), 0, 0,
						this, SLOT(slotShowStellaDocumentation()),
						actionCollection(), "documentation");
	(void) new KAction( i18n("Reload"), "reload", KStdAccel::shortcut(KStdAccel::Reload),
						this, SLOT(slotEmulatorReload()),
						actionCollection(), "reload");
	KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());

	// Settings menu
	(void) new KAction( i18n("Emulator Defaults"), 0, 0,
						this, SLOT(slotEmulatorDefaults()),
						actionCollection(), "emulator_defaults");
	(void) new KAction( i18n("Game Defaults"), 0, 0,
						this, SLOT(slotGameDefaults()),
						actionCollection(), "game_defaults");
	KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), actionCollection());

	// View menu
	actionAllRoms =	new KToggleAction( i18n("All Roms"), 0, 0, this,
					SLOT(slotViewAllRoms()), actionCollection(),
					"view_all_roms");
	actionAvailableRoms = new KToggleAction( i18n("Available Roms"), 0, 0, this,
						  SLOT(slotViewAvailableRoms()), actionCollection(),
 						  "view_available_roms");
	actionUnavailableRoms =	new KToggleAction( i18n("Unavailable Roms"), 0, 0, this,
							SLOT(slotViewUnavailableRoms()), actionCollection(),
							"view_unavailable_roms");
	actionRarity = new KToggleAction( i18n("Rarity"), 0, 0, this,
				   SLOT(slotToggleRarity()), actionCollection(),
				   "toggle_rarity");
	actionManufacturer = new KToggleAction( i18n("Manufacturer"), 0, 0, this,
						 SLOT(slotToggleManufacturer()), actionCollection(),
						 "toggle_manufacturer");
	actionPreviewWindow = new KToggleAction( i18n("Preview Window"), 0, 0, this,
						  SLOT(slotTogglePreview()), actionCollection(),
						  "toggle_preview");
	actionDebugWindow = new KToggleAction( i18n("Debug Window"), 0, 0, this,
						SLOT(slotToggleGameInfo()), actionCollection(),
						"toggle_debug");

	// Game menu
	(void) new KAction( i18n("Settings"), 0, this,
						SLOT(slotSettings()), actionCollection(),
						"game_settings");
	(void) new KAction( i18n("Emulate"), 0, this,
						SLOT(slotEmulate()),actionCollection(),
						"game_emulate");

	config->setGroup( "General Options" );
	actionRarity->setChecked( config->readBoolEntry( "Rarity", true ) );
	actionManufacturer->setChecked( config->readBoolEntry( "Manufacturer", true ) );
	actionPreviewWindow->setChecked( config->readBoolEntry( "Preview", true ) );
	actionDebugWindow->setChecked( config->readBoolEntry( "Game Info", true ) );

	createGUI();
}

/*
void KStella::initToolBar()
{
	toolBar()->insertButton( "", ID_FILE_STATISTICS, true, i18n("Statistics") );

	connect(toolBar(), SIGNAL(clicked(int)), SLOT(commandCallback(int)) );
}
    */

void KStella::initView( void )
{
	// create the main widget for the window
	splitter = new QSplitter( Qt::Horizontal, this );

	// add the game list and preview window
	view = new GameListView( splitter, "view" );
	preview = new PreviewWindow( splitter, "preview" );

	config->setGroup( "General Options" );
	QValueList<int> splitterSizes = config->readIntListEntry( "Splitter Sizes" );
	splitter->setSizes( splitterSizes );
	splitter->setResizeMode( preview, QSplitter::Stretch );
	showPreview( config->readBoolEntry( "Preview" ) );

	// finally, show the widgets onscreen
	setCentralWidget( splitter );
	setCaption( "Stella Frontend" );
	this->show();
}


void KStella::saveOptions( void )
{
	config->setGroup( "General Options" );
	config->writeEntry( "Splitter Sizes", splitter->sizes() );
	config->writeEntry( "Rom View", (int)romView() );
	config->writeEntry( "Rarity", actionRarity->isChecked() );
	config->writeEntry( "Manufacturer", actionManufacturer->isChecked() );
	config->writeEntry( "Preview", actionPreviewWindow->isChecked() );
	config->writeEntry( "Game Info", actionDebugWindow->isChecked() );
}


void KStella::showPreview( bool toggle )
{
	if( toggle )
		preview->show();
	else
		preview->hide();
}


/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KStella::slotFileQuit()
{
	if( this )
		delete this;
	kapp->quit();
}


void KStella::slotEmulatorDefaults( void )
{
	Setup setup;
	setup.exec();
}


void KStella::slotGameDefaults( void )
{
	Settings settings( this, "settings", "Stella - Game Defaults", "Game Defaults" );
	settings.exec();
}


void KStella::slotEmulatorReload( void )
{
	doc->reload();
	view->update();
}


void KStella::slotViewAllRoms( void )
{
	setRomView( KStella::ALL );
	///////////////////////////////////////////////////////////////////
	// turn view ALL roms on
	actionAllRoms->setChecked( true );
	actionAvailableRoms->setChecked( false );
	actionUnavailableRoms->setChecked( false );

	view->update();
}


void KStella::slotViewAvailableRoms( void )
{
	setRomView( KStella::AVAILABLE );
	///////////////////////////////////////////////////////////////////
	// turn view AVAILABLE roms on
	actionAllRoms->setChecked( false );
	actionAvailableRoms->setChecked( true );
	actionUnavailableRoms->setChecked( false );

	view->update();
}


void KStella::slotViewUnavailableRoms( void )
{
	setRomView( KStella::UNAVAILABLE );
	///////////////////////////////////////////////////////////////////
	// turn view UNAVAILABLE games on
	actionAllRoms->setChecked( false );
	actionAvailableRoms->setChecked( false );
	actionUnavailableRoms->setChecked( true );

	view->update();
}


void KStella::slotToggleRarity( void )
{
	bool show = actionRarity->isChecked();
	view->showRarity( show );
	view->update();
}


void KStella::slotToggleManufacturer( void )
{
	bool show = actionManufacturer->isChecked();
	view->showManufacturer( show );
	view->update();
}


void KStella::slotToggleGameInfo( void )
{
	bool gameinfo = actionDebugWindow->isChecked();
	config->setGroup( "General Options" );
	config->writeEntry( "Game Info", gameinfo );
}


void KStella::slotTogglePreview( void )
{
	bool show = actionPreviewWindow->isChecked();
	showPreview( show );
}


void KStella::slotSettings( void )
{
	Game* g = view->currentGame();
	if( g == (Game*) NULL )
		return;

	Settings settings( (QWidget*) 0, "settings", g->name(), g->md5() );
	settings.exec();
}


void KStella::slotEmulate( void )
{
	QString gameArgs, emulatorArgs;

	Game* g = view->currentGame();
	if( (g == (Game*) NULL) || !g->isAvailable() )
	{
		QMessageBox::information( this, "KStella", i18n("No ROM available.") );
		return;
	}

	// Check to see if we should echo the full commandline
	config->setGroup( "General Options" );
	bool gameinfo = config->readBoolEntry( "Game Info", true );

	config->setGroup( "Emulator Defaults" );
	QString binary = config->readEntry( "Binary" );
	emulatorArgs = config->readEntry( "EmulatorArgs" );

	// try to read the game settings before the defaults
	config->setGroup( g->md5() );
	gameArgs = config->readEntry( "GameArgs" );
	if( gameArgs.isNull() )
	{
		config->setGroup( "Game Defaults" );
		gameArgs = config->readEntry( "GameArgs" );
	}

	// run the emulator for the current item/game
	// Prepend the actual stella executable - it better be in the user PATH
    QString command = binary + " " + gameArgs + " " + emulatorArgs + " ";
	if( gameinfo )
		cerr << "Command: " << command << endl;
    KRun::run( command, QStringList( g->rom() ) );
}


void KStella::slotShowStatistics( void )
{
	QMessageBox::information( this, "Game Statistics", i18n(doc->gameStatistics()) );
}


void KStella::slotShowStellaDocumentation( void )
{
	config->setGroup( "Emulator Defaults" );
	QString doc = config->readEntry( "DocDir" );
	doc += "/stella.html";

	// check to see if the documentation is actually there
	if( QFile::exists( doc ) )
		new KRun( KURL(doc), 0, true, true );
	else
		QMessageBox::information( this, "KStella", i18n("Stella documentation not available.") );
}


void KStella::slotConfigureKeys( void )
{
	KKeyDialog::configure( actionCollection(), this );
}


void KStella::slotChangeNote( Game* g )
{
	if( g->note().length() > 1 )
	{
		QString s = "Note:   " + g->note();
		statusBar()->message( s );
	}
	else
		statusBar()->clear();
}
