/***************************************************************************
 *   Copyright (C) 2004 by Stephen Anthony                                 *
 *   stephena@users.sourceforge.net                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef PREVIEWWINDOW_H
#define PREVIEWWINDOW_H

#include <kconfig.h>

#include <qtabwidget.h>
#include <qtextbrowser.h>
#include <qwidget.h>
#include <qpixmap.h>
#include <qframe.h>

#include "game.h"
#include "preview.h"


/** This class contains the snapshot and label preview widgets in a tab view.
	The previews can be switched on or off as desired.

	@author Stephen Anthony
*/
class PreviewWindow : public QWidget
{
  Q_OBJECT

	public:
		PreviewWindow( QWidget* parent = 0, const char* name = 0 );
		~PreviewWindow( void );

	public slots:
		/** indicates that the tab should be changed, and was probably
			initiated by the GameListView */
		void slotChangeTab( int direction );

		/** indicates that the current game has been updated */
		void slotSetCurrentGame( Game* g );

	private slots:
		/** internal method called when the tab has changed by clicking the mouse */
		void slotTabChanged( QWidget* );

		/** this is called after the last resize event has been processed */
		void slotSizeChanged( void );

	private:
		/** re-implemented to take care of multiple resize events */
		void resizeEvent( QResizeEvent* e );

		/** update the snapshot tab */
		void doSnapshot( void );

		/** update the label tab */
		void doLabel( void );

		/** update the manual tab */
		void doManual( void );

	private:
		/** the configuration object of the application */
		KConfig* config;

		/** used to create the snapshot and label tabs */
		QTabWidget* tabs;

		/** this widget shows game snapshots */
		Preview* snapshot;

		/** this widget shows cartridge labels */
		Preview* label;

		/** this widget shows game manuals */
		QTextBrowser* html;

		/** indicates how many tabs are contained in the preview window */
		int numTabs;

		/** indicates the current game selected */
		Game* currentGame;

		/** used to delay processing of resizes for a while */
		QTimer* resizeTimer;

		/** static pixmap which is displayed when no preview is available */
		QPixmap* noPreview;
};

#endif
